function IDM:color_from_suspicion(amount)
	local color = Vector3(60, 220, 150) / 255
	if amount >= 0.8 then
		color = Vector3(160, 10, 0) / 255
	elseif amount >= 0.6 then
		color = Vector3(255, 45, 0) / 255
	elseif amount >= 0.4 then
		color = Vector3(255, 190, 0) / 255
	elseif amount >= 0.2 then
		color = Vector3(60, 195, 0) / 255
	end
	return color
end

function IDM:cooldown_from_suspicion(amount)
	local cooldown = 1
	if amount >= 0.8 then
		cooldown = 5
	elseif amount >= 0.6 then
		cooldown = 4
	elseif amount >= 0.4 then
		cooldown = 3
	elseif amount >= 0.2 then
		cooldown = 2
	end
	return cooldown
end

function IDM:cooldown_suspicion(time)
	if IDM._settings.cooldown_enabled and time > IDM.cooldown_time then
		for unit_id, suspicion in pairs(IDM.suspicion) do
			if suspicion.previous == 0 then
				local cumulative_status = suspicion.cumulative
				if cumulative_status > 0 then
					if suspicion.cooldown == 0 then
						IDM.suspicion[unit_id].cumulative = cumulative_status - 0.1 > 0 and cumulative_status - 0.1 or 0
						IDM:send_suspicion(unit_id, IDM.suspicion[unit_id].cumulative)
						for _, enemy in pairs(managers.enemy:all_enemies()) do
							if enemy.unit:id() == unit_id then
								if enemy.unit:contour() then
									enemy.unit:contour():replace_contour()
								end
							end
						end
					else
						IDM.suspicion[unit_id].cooldown = suspicion.cooldown - 1
					end
				end
			end
		end
		IDM.cooldown_time = time + 10
	end
end

function IDM:send_suspicion(unit_id, cumulative_status)
	if Network:multiplayer() and table.size(managers.network:session():peers()) > 0 then
		LuaNetworking:SendToPeers('IDM#27400_Network_update_suspicion', tostring(unit_id) .. '|' .. tostring(cumulative_status))
	end
end

Hooks:AddHook('NetworkReceivedData', 'IDM#27400_Network_receive_suspicion',
function(sender, id, stream)
	if Network:is_client() and id == 'IDM#27400_Network_update_suspicion' then
		local data = string.split(stream, '|')
		local unit_id = tonumber(data[1])
		local cumulative_status = tonumber(data[2])
		if not table.has(IDM.suspicion, unit_id) then
			IDM.suspicion[unit_id] = {}
		end
		IDM.suspicion[unit_id].cumulative = cumulative_status
		for _, enemy in pairs(managers.enemy:all_enemies()) do
			if enemy.unit:id() == unit_id then
				if enemy.unit:contour() then
					enemy.unit:contour():replace_contour()
				end
			end
		end	
	end
end)

function PlayerMovement:on_suspicion(observer_unit, status)
	if Network:is_server() then
		local visible_status = nil
		if managers.groupai:state():whisper_mode() and not managers.groupai:state():stealth_hud_disabled() then
			visible_status = status
		else
			visible_status = false
		end
		self._suspicion = self._suspicion or {}
		if visible_status == false or visible_status == true then
			self._suspicion[observer_unit:key()] = nil
			if IDM._settings.mod_enabled and table.has(IDM.suspicion, observer_unit:id()) and visible_status == false then
				IDM.suspicion[observer_unit:id()].previous = 0
			end
			if not next(self._suspicion) then
				self._suspicion = nil
			end
			if visible_status and observer_unit:movement() and not observer_unit:movement():cool() and TimerManager:game():time() - observer_unit:movement():not_cool_t() > 1 then
				self._suspicion_ratio = false
				self:_feed_suspicion_to_hud()
				return
			end
		elseif type(visible_status) == 'number' and (not observer_unit:movement() or observer_unit:movement():cool()) then
			if IDM._settings.mod_enabled and table.has(IDM.suspicion, observer_unit:id()) then
				local cumulative_status = IDM.suspicion[observer_unit:id()].cumulative
				local previous_status = IDM.suspicion[observer_unit:id()].previous
				local increase_status = visible_status - previous_status
				if increase_status > 0 then
					cumulative_status = cumulative_status + increase_status
				end
				previous_status = visible_status
				self._suspicion[observer_unit:key()] = cumulative_status >= 1 and 1 or cumulative_status
				IDM.suspicion[observer_unit:id()].cumulative = cumulative_status
				IDM.suspicion[observer_unit:id()].previous = previous_status
				IDM.suspicion[observer_unit:id()].cooldown = IDM:cooldown_from_suspicion(cumulative_status)
				if increase_status > 0 then
					if observer_unit:contour() then
						observer_unit:contour():replace_contour()
					end
				end
				IDM:send_suspicion(observer_unit:id(), cumulative_status)
				if cumulative_status >= 1 then		
					managers.groupai:state():on_criminal_suspicion_progress(self._unit, observer_unit, true)
					managers.groupai:state():criminal_spotted(self._unit)
					observer_unit:movement():set_cool(false)
					self:_calc_suspicion_ratio_and_sync(observer_unit, 1)
					self:on_suspicion(observer_unit, true)
					return
				end
			else
				self._suspicion[observer_unit:key()] = visible_status
			end
		else
			return
		end
		self:_calc_suspicion_ratio_and_sync(observer_unit, visible_status)
	else
		self._suspicion_ratio = status
	end
	self:_feed_suspicion_to_hud()
end

Hooks:PostHook(PlayerMovement, 'update', 'IDM#27400_PlayerMovement_update',
function(self, unit, t, dt)
	if IDM._settings.mod_enabled and Network:is_server() and managers.groupai:state():whisper_mode() then
		IDM:cooldown_suspicion(t)
	end
end)